@@ -1,21 +1,23 @@
 from functools import cmp_to_key
 from typing import Any, Callable, List, Tuple
 
+from m.color import color
+
 from .types import Configuration, ExitCode, ProjectStatus, RuleInfo
 
 
-def _compare_rules(x: RuleInfo, y: RuleInfo) -> int:
-    diff = x.found - y.found or x.allowed - y.allowed
+def _compare_rules(rule_a: RuleInfo, rule_b: RuleInfo) -> int:
+    diff = rule_a.found - rule_b.found or rule_a.allowed - rule_b.allowed
     if not diff:
-        return 1 if x.rule_id < y.rule_id else -1
+        return 1 if rule_a.rule_id < rule_b.rule_id else -1
     return diff
 
 
 def _compare_rule_items(
-    x: Tuple[str, RuleInfo],
-    y: Tuple[str, RuleInfo],
+    rule_a: Tuple[str, RuleInfo],
+    rule_b: Tuple[str, RuleInfo],
 ) -> int:
-    return _compare_rules(x[1], y[1])
+    return _compare_rules(rule_a[1], rule_b[1])
 
 
 def _align(token: Any, alignment: str) -> Callable[[int], str]:
@@ -36,7 +38,9 @@ def rule_info_str(
         A string representation of the rule info.
     """
     allowed = 'IGNORED' if rule.ignored else f'allowed {rule.allowed}'
-    buffer = [f'{rule.rule_id} (found {rule.found}, {allowed}):']
+    rule_id = color(f'{{red}}{rule.rule_id}{{end}}')
+    total_found = color(f'{{red}}{rule.found}{{end}}')
+    buffer = [f'{rule_id} (found {total_found}, {allowed}):']
     violations = (
         rule.violations
         if config.max_lines == -1
@@ -47,7 +51,8 @@ def rule_info_str(
         msg, *rest = violation.message.splitlines()
         line = violation.line
         column = violation.column
-        buffer.append(f'  {file_path}:{line}:{column} - {msg}')
+        file_loc = color(f'{{gray}}{file_path}:{line}:{column}{{end}}')
+        buffer.append(f'  {file_loc} - {msg}')
         if rest and config.full_message:
             buffer.extend([f'    {x}' for x in rest])
     if -1 < config.max_lines < len(rule.violations):
@@ -103,9 +108,16 @@ def project_status_str(
             if rule.ignored
         ]
         if project.total_found > 0:
-            buffer.append(f'project has {project.total_found} errors to clear')
+            buffer.append(
+                color(
+                    '{gray}project has ',
+                    f'{{red}}{project.total_found} errors',
+                    '{gray} to clear',
+                ),
+            )
         else:
-            buffer.append('no errors found')
+            msg = 'no errors found'
+            buffer.append(color(f'{{bold_green}}{msg}'))
         return '\n'.join(buffer)
 
     buffer = [
@@ -114,10 +126,10 @@ def project_status_str(
         if rule.found > rule.allowed
     ]
 
-    buffer.append('FILES:')
+    buffer.append(color('{bold}FILES:'))
     by_file = sorted(project.files.items(), key=lambda t: len(t[1]))
     buffer.extend([
-        f'  {file_name}: found {total}'
+        color(f'  {{gray}}{file_name}:{{end}} found {total}')
         for file_name, violations in by_file
         for total in (len(violations), )
     ])
@@ -137,7 +149,14 @@ def project_status_str(
     )
     buffer.append(format_row(['RULES', 'FOUND', 'ALLOWED'], widths, 'lll'))
     buffer.extend([
-        format_row([rule_id, rule.found, rule.allowed], widths, 'lrr')
+        (
+            color(
+                '{gray}',
+                format_row([rule_id, rule.found, rule.allowed], widths, 'lrr'),
+            )
+            if rule.found == rule.allowed
+            else format_row([rule_id, rule.found, rule.allowed], widths, 'lrr')
+        )
         for rule_id, rule in key_rule
         if not rule.ignored
     ])
